/*
Controls:

APPS: Draw
STO->: Erase
VAR: Slow down/hide cursor
MODE: Clear screen
TOOL: Pause
NXT: Unpause
F1: Speed Up
F2: Slow Down
ON: Quit
*/


#include <hpgcc49.h>
#include <hpgraphics.h>

#define HEIGHT 80
#define WIDTH 131


char count[HEIGHT*WIDTH];
char state[HEIGHT*WIDTH];
char newcount[HEIGHT*WIDTH];
int curx, cury;
int paused;


// UPDATE LIFE COUNT FOR NEIGHBORS

void add_neighbors(int j,int i,int value)
{
int offset0;
int leftadd,rightadd,topadd,botadd;

if(i==0) leftadd=WIDTH-1; else leftadd=-1;
if(i==WIDTH-1) rightadd=-WIDTH+1; else rightadd=1;
if(j==0) topadd=(HEIGHT-1)*WIDTH; else topadd=-WIDTH;
if(j==HEIGHT-1) botadd=-(HEIGHT-1)*WIDTH; else botadd=WIDTH;


offset0=j*WIDTH+i;

// TOP-LEFT
newcount[offset0+topadd+leftadd]+=value;
// TOP
newcount[offset0+topadd]+=value;
// TOP-RIGHT
newcount[offset0+topadd+rightadd]+=value;
// LEFT
newcount[offset0+leftadd]+=value;
// RIGHT
newcount[offset0+rightadd]+=value;
// BOT-LEFT
newcount[offset0+botadd+leftadd]+=value;
// BOT
newcount[offset0+botadd]+=value;
// BOT-RIGHT
newcount[offset0+botadd+rightadd]+=value;


}


// DETERMINE NEW STATE AFTER ONE TICK

void play_tick()
{
int i,j;
int offset;
// COPY COUNT TO TEMPORARY STORAGE
memmove(newcount,count,sizeof(char)*HEIGHT*WIDTH);

offset=0;

for(j=0;j<HEIGHT;++j)
{
for(i=0;i<WIDTH;++i)
{
if(state[offset]) {
// IF ALIVE, CHECK IF IT SHOULD DIE
if( (count[offset]|1)!=3 ) {
// KILL IT
state[offset]=0;
hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE);
hpg_draw_pixel(i,j);
// UPDATE COUNT FOR ALL NEIGHBORS
add_neighbors(j,i,-1);

}

}
else {
// IF DEAD, CHECK IF IT SHOULD COME TO LIFE
if(count[offset]==3) {
state[offset]=1;
hpg_set_color(hpg_stdscreen,HPG_COLOR_BLACK);
hpg_draw_pixel(i,j);
// UPDATE NEIGHBORS COUNT
add_neighbors(j,i,1);
}
}
++offset;
}
}

// MOVE BACK TO PERMANENT STORAGE
memmove(count,newcount,sizeof(char)*WIDTH*HEIGHT);

}

// SET INITIAL STATE
void calc_count()
{
int i,j;
int offset;
// COPY COUNT TO TEMPORARY STORAGE
memset((void *)newcount,0,sizeof(char)*HEIGHT*WIDTH);

offset=0;

for(j=0;j<HEIGHT;++j)
{
for(i=0;i<WIDTH;++i)
{
if(state[offset]) { add_neighbors(j,i,1); hpg_set_color(hpg_stdscreen,HPG_COLOR_BLACK); hpg_draw_pixel(i,j); }
else { hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE); hpg_draw_pixel(i,j); } 
++offset;
}
}

// MOVE BACK TO PERMANENT STORAGE
memmove(count,newcount,sizeof(char)*WIDTH*HEIGHT);

}



int main()
{

paused=0;

hpg_clear();

// SET TO RANDOM STATE
hpg_draw_text("Conway's Game of Life",20,30);
curx= WIDTH/2;
cury= HEIGHT/2;

// GET STATE MATRIX

int i,j,offset;
offset=0;
for(j=0;j<HEIGHT;++j)
{
for(i=0;i<WIDTH;++i)
{
if(hpg_get_pixel(hpg_stdscreen,i,j)) state[offset]=1;
else state[offset]=0;
++offset;
}
}


// INITIATE CONWAY'S ITERATOR
calc_count();

while(!keyb_isON())
{

if(keyb_isKeyPressed(6, 4))
{
for(j=0;j<(HEIGHT*WIDTH);++j)
{
count[j]=0;
newcount[j]=0;
state[j]=0;
}
hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE);
hpg_fill_rect(0,0,130,79);
}

if(keyb_isKeyPressed(5,6))
{
if(state[curx+(cury*WIDTH)]==0)
{
memmove(newcount,count,sizeof(char)*HEIGHT*WIDTH);
state[curx+(cury*WIDTH)]=1;
hpg_set_color(hpg_stdscreen,HPG_COLOR_BLACK);
hpg_draw_pixel(curx, cury);
add_neighbors(cury, curx,1);
memmove(count,newcount,sizeof(char)*WIDTH*HEIGHT);
}
sys_sleep(30);
if(keyb_isKeyPressed(6,6))
{
sys_sleep(210);
}
}
else
{
if(keyb_isKeyPressed(7,0))
{
if(state[curx+(cury*WIDTH)]==1)
{
memmove(newcount,count,sizeof(char)*HEIGHT*WIDTH);
state[curx+(cury*WIDTH)]=0;
hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE);
hpg_draw_pixel(curx, cury);
add_neighbors(cury, curx,-1);
memmove(count,newcount,sizeof(char)*WIDTH*HEIGHT);
}
sys_sleep(30);
if(keyb_isKeyPressed(6,6))
{
sys_sleep(210);
}
}
else
{
if(paused==0)
{
play_tick();
}
}
}

hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE);

if(state[curx+1+(cury*WIDTH)]==0)
{
hpg_draw_pixel(curx+1, cury);
}
if(state[curx+2+(cury*WIDTH)]==0)
{
hpg_draw_pixel(curx+2, cury);
}
if(state[curx-1+(cury*WIDTH)]==0)
{
hpg_draw_pixel(curx-1, cury);
}
if(state[curx-2+(cury*WIDTH)]==0)
{
hpg_draw_pixel(curx-2, cury);
}
if(state[curx+((cury+1)*WIDTH)]==0)
{
hpg_draw_pixel(curx, cury+1);
}
if(state[curx+((cury+2)*WIDTH)]==0)
{
hpg_draw_pixel(curx, cury+2);
}
if(state[curx+((cury-1)*WIDTH)]==0)
{
hpg_draw_pixel(curx, cury-1);
}
if(state[curx+((cury-2)*WIDTH)]==0)
{
hpg_draw_pixel(curx, cury-2);
}

if(keyb_isRight())
{
curx++;
}
if(keyb_isLeft())
{
curx--;
}
if(keyb_isUp())
{
cury--;
}
if(keyb_isDown())
{
cury++;
}

if (curx<0)
{
curx = 0;
}
if (curx>130)
{
curx = 130;
}
if (cury<0)
{
cury = 0;
}
if (cury>79)
{
cury = 79;
}

if(!keyb_isKeyPressed(6,6))
{
hpg_set_color(hpg_stdscreen,HPG_COLOR_BLACK);
hpg_draw_pixel(curx+1, cury);
hpg_draw_pixel(curx+2, cury);
hpg_draw_pixel(curx-1, cury);
hpg_draw_pixel(curx-2, cury);
hpg_draw_pixel(curx, cury+1);
hpg_draw_pixel(curx, cury+2);
hpg_draw_pixel(curx, cury-1);
hpg_draw_pixel(curx, cury-2);
}
if(keyb_isKeyPressed(6,5))
{
paused=1;
}
if(keyb_isKeyPressed(7,1))
{
paused=0;
}
if(keyb_isKeyPressed(5,0))
{
sys_slowOff();
}
if(keyb_isKeyPressed(5,1))
{
sys_slowOn();
}
if(paused==1)
{
sys_sleep(40);
}
}
sys_slowOn();
return 0;
}

